package Rose::HTML::Object::Error::Localized;

use strict;

use Rose::HTML::Object::Error;
use Rose::HTML::Object::Message::Localized;
our @ISA = qw(Rose::HTML::Object::Message::Localized Rose::HTML::Object::Error);

our $VERSION = '0.54';

1;
